@@ -1,12 +1,12 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 import copy
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
 from trytond.report import Report
-from trytond.pyson import Eval, If, In, Get
+from trytond.pyson import Eval, If
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 
 class Company(ModelSQL, ModelView):
@@ -34,7 +34,7 @@ class Company(ModelSQL, ModelView):
         })
 
     def copy(self, ids, default=None):
-        party_obj = self.pool.get('party.party')
+        party_obj = Pool().get('party.party')
 
         int_id = False
         if isinstance(ids, (int, long)):
@@ -56,7 +56,7 @@ class Company(ModelSQL, ModelView):
     def write(self, ids, vals):
         res = super(Company, self).write(ids, vals)
         # Restart the cache on the domain_get method
-        self.pool.get('ir.rule').domain_get.reset()
+        Pool().get('ir.rule').domain_get.reset()
         return res
 
 Company()
@@ -84,7 +84,8 @@ class User(ModelSQL, ModelView):
     companies = fields.Function(fields.One2Many('company.company', None,
         'Current Companies'), 'get_companies')
     employee = fields.Many2One('company.employee', 'Employee',
-            domain=[('company', 'child_of', [Eval('main_company')], 'parent')])
+        domain=[('company', 'child_of', [Eval('main_company')], 'parent')],
+        depends=['main_company'])
 
     def __init__(self):
         super(User, self).__init__()
@@ -104,7 +105,7 @@ class User(ModelSQL, ModelView):
         return self.default_main_company()
 
     def get_companies(self, ids, name):
-        company_obj = self.pool.get('company.company')
+        company_obj = Pool().get('company.company')
         res = {}
         company_childs = {}
         for user in self.browse(ids):
@@ -130,14 +131,15 @@ class User(ModelSQL, ModelView):
         res = super(User, self).get_status_bar(ids, name)
         for user in self.browse(ids):
             if user.company:
-                res[user.id] += ' ' + user.company.name
+                res[user.id] += ' - %s [%s]' % (user.company.name,
+                    user.company.currency.name)
         return res
 
     def on_change_main_company(self, vals):
         return {'company': vals.get('main_company', False)}
 
     def check_company(self, ids):
-        company_obj = self.pool.get('company.company')
+        company_obj = Pool().get('company.company')
         for user in self.browse(ids):
             if user.main_company:
                 companies = company_obj.search([
@@ -159,7 +161,7 @@ class User(ModelSQL, ModelView):
         return res
 
     def get_preferences_fields_view(self):
-        company_obj = self.pool.get('company.company')
+        company_obj = Pool().get('company.company')
 
         user = self.browse(Transaction().user)
 
@@ -167,19 +169,52 @@ class User(ModelSQL, ModelView):
         res = copy.deepcopy(res)
         return res
 
+    def read(self, ids, fields_names=None):
+        company_obj = Pool().get('company.company')
+        user_id = Transaction().user
+        if user_id == 0 and 'user' in Transaction().context:
+            user_id = Transaction().context['user']
+        result = super(User, self).read(ids, fields_names=fields_names)
+        if (fields_names
+                and 'company' in fields_names
+                and 'company' in Transaction().context):
+            values = None
+            if isinstance(ids, (int, long)):
+                if int(user_id) == ids:
+                    values = result
+            else:
+                if int(user_id) in ids:
+                    for vals in result:
+                        if vals['id'] == int(user_id):
+                            values = vals
+                            break
+            if values:
+                main_company_id = values.get('main_company')
+                if not main_company_id:
+                    main_company_id = self.read(user_id,
+                        ['main_company'])['main_company']
+                companies = company_obj.search([
+                    ('parent', 'child_of', [main_company_id]),
+                ])
+                company_id = Transaction().context['company']
+                if ((company_id and company_id in companies)
+                        or not company_id):
+                    values['company'] = company_id
+        return result
+
 User()
 
 
 class Property(ModelSQL, ModelView):
     _name = 'ir.property'
     company = fields.Many2One('company.company', 'Company',
-            domain=[
-                ('id', If(In('company', Eval('context', {})), '=', '!='),
-                    Get(Eval('context', {}), 'company', 0)),
+        domain=[
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
+                Eval('context', {}).get('company', 0)),
             ])
 
     def _set_values(self, name, model, res_id, val, field_id):
-        user_obj = self.pool.get('res.user')
+        user_obj = Pool().get('res.user')
         user_id = Transaction().user
         if user_id == 0:
             user_id = Transaction().context.get('user', user_id)
@@ -202,9 +237,9 @@ Property()
 class Sequence(ModelSQL, ModelView):
     _name = 'ir.sequence'
     company = fields.Many2One('company.company', 'Company',
-            domain=[
-                ('id', If(In('company', Eval('context', {})), '=', '!='),
-                    Get(Eval('context', {}), 'company', 0)),
+        domain=[
+            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
+                Eval('context', {}).get('company', 0)),
             ])
 
     def __init__(self):
@@ -265,8 +300,8 @@ class CompanyConfig(Wizard):
     }
 
     def _add(self, data):
-        company_obj = self.pool.get('company.company')
-        user_obj = self.pool.get('res.user')
+        company_obj = Pool().get('company.company')
+        user_obj = Pool().get('res.user')
 
         company_id = company_obj.create(data['form'])
         user_ids = user_obj.search([
@@ -284,7 +319,7 @@ CompanyConfig()
 class CompanyReport(Report):
 
     def parse(self, report, objects, datas, localcontext=None):
-        user = self.pool.get('res.user').browse(Transaction().user)
+        user = Pool().get('res.user').browse(Transaction().user)
         if localcontext is None:
             localcontext = {}
         localcontext['company'] = user.company
